@@ -5,9 +5,14 @@ import numpy as np
 import pandas as pd
 import matplotlib.pylab as plt
 import mhkit.river as river
+import netCDF4
+from numpy.testing import assert_array_almost_equal
+from pandas.testing import assert_frame_equal
+import scipy.interpolate as interp
+
 
 testdir = dirname(abspath(__file__))
-datadir = normpath(join(testdir,relpath('../../examples/data/river')))
+datadir = normpath(join(testdir,'..','..','examples','data','river'))
 
 
 class TestPerformance(unittest.TestCase):
@@ -71,7 +76,7 @@ class TestResource(unittest.TestCase):
     @classmethod
     def setUpClass(self):
         self.data = pd.read_csv(join(datadir, 'tanana_discharge_data.csv'), index_col=0, 
-                           parse_dates=True)
+                            parse_dates=True)
         self.data.columns = ['Q']
               
         self.results = pd.read_csv(join(datadir, 'tanana_test_results.csv'), index_col=0, 
@@ -233,7 +238,10 @@ class TestIO(unittest.TestCase):
 
     @classmethod
     def setUpClass(self):
-        pass
+        d3ddatadir = normpath(join(datadir,'d3d'))
+        
+        filename= 'turbineTest_map.nc'
+        self.d3d_flume_data = netCDF4.Dataset(join(d3ddatadir,filename))
         
     @classmethod
     def tearDownClass(self):
@@ -241,14 +249,14 @@ class TestIO(unittest.TestCase):
     
     def test_load_usgs_data_instantaneous(self):
         file_name = join(datadir, 'USGS_08313000_Jan2019_instantaneous.json')
-        data = river.io.read_usgs_file(file_name)
+        data = river.io.usgs.read_usgs_file(file_name)
         
         self.assertEqual(data.columns, ['Discharge, cubic feet per second'])
         self.assertEqual(data.shape, (2972, 1)) # 4 data points are missing
         
     def test_load_usgs_data_daily(self):
         file_name = join(datadir, 'USGS_08313000_Jan2019_daily.json')
-        data = river.io.read_usgs_file(file_name)
+        data = river.io.usgs.read_usgs_file(file_name)
 
         expected_index = pd.date_range('2019-01-01', '2019-01-31', freq='D')
         self.assertEqual(data.columns, ['Discharge, cubic feet per second'])
@@ -257,7 +265,7 @@ class TestIO(unittest.TestCase):
 
 
     def test_request_usgs_data_daily(self):
-        data=river.io.request_usgs_data(station="15515500",
+        data=river.io.usgs.request_usgs_data(station="15515500",
                             parameter='00060',
                             start_date='2009-08-01',
                             end_date='2009-08-10',
@@ -267,7 +275,7 @@ class TestIO(unittest.TestCase):
     
    
     def test_request_usgs_data_instant(self):
-        data=river.io.request_usgs_data(station="15515500",
+        data=river.io.usgs.request_usgs_data(station="15515500",
                             parameter='00060',
                             start_date='2009-08-01',
                             end_date='2009-08-10',
@@ -277,6 +285,88 @@ class TestIO(unittest.TestCase):
         self.assertEqual(data.shape, (10*24*4, 1))
 
 
+    def test_layer_data(self): 
+        data=self.d3d_flume_data
+        variable= 'ucx'
+        layer=2 
+        time_index= 3
+        layer_data= river.io.d3d.get_layer_data(data, variable, layer, time_index)
+        layer_compare = 2
+        time_index_compare= 4
+        layer_data_expected= river.io.d3d.get_layer_data(data,
+                                                        variable, layer_compare,
+                                                        time_index_compare)
+       
+        assert_array_almost_equal(layer_data.x,layer_data_expected.x, decimal = 2)
+        assert_array_almost_equal(layer_data.y,layer_data_expected.y, decimal = 2)
+        assert_array_almost_equal(layer_data.v,layer_data_expected.v, decimal= 2)
+        
+        
+    def test_create_points(self):
+        x=np.linspace(1, 3, num= 3)
+        y=np.linspace(1, 3, num= 3)
+        z=1 
+        points= river.io.d3d.create_points(x,y,z)
+        
+        x=[1,2,3,1,2,3,1,2,3]
+        y=[1,1,1,2,2,2,3,3,3]
+        z=[1,1,1,1,1,1,1,1,1]
+        
+        points_array= np.array([ [x_i, y_i, z_i] for x_i, y_i, z_i in zip(x, y, z)]) 
+        points_expected= pd.DataFrame(points_array, columns=('x','y','z'))
+        assert_array_almost_equal(points, points_expected,decimal = 2)  
+        
+        
+    def test_get_all_data_points(self): 
+        data=self.d3d_flume_data
+        variable= 'ucx'
+        time_step= 3
+        output = river.io.d3d.get_all_data_points(data, variable, time_step)
+        size_output = np.size(output) 
+        time_step_compair=4
+        output_expected= river.io.d3d.get_all_data_points(data, variable, time_step_compair)
+        size_output_expected= np.size(output_expected)
+        self.assertEqual(size_output, size_output_expected)
+ 
+    
+    def test_unorm(self): 
+        x=np.linspace(1, 3, num= 3)
+        y=np.linspace(1, 3, num= 3)
+        z=np.linspace(1, 3, num= 3)
+        unorm = river.io.d3d.unorm(x,y,z)
+        unorm_expected= [np.sqrt(1**2+1**2+1**2),np.sqrt(2**2+2**2+2**2), np.sqrt(3**2+3**2+3**2)]
+        assert_array_almost_equal(unorm, unorm_expected, decimal = 2) 
+    
+    def test_turbulent_intensity(self): 
+        data=self.d3d_flume_data
+        time_step= -1
+        x_test=np.linspace(1, 17, num= 10)
+        y_test=np.linspace(3, 3, num= 10)
+        z_test=np.linspace(1, 1, num= 10)
+       
+        test_points = np.array([ [x, y, z] for x, y, z in zip(x_test, y_test, z_test)])
+        points= pd.DataFrame(test_points, columns=['x','y','z'])
+        
+        TI= river.io.d3d.turbulent_intensity(data, points, time_step)
+
+        TI_vars= ['turkin1', 'ucx', 'ucy', 'ucz']
+        TI_data_raw = {}
+        for var in TI_vars:
+            #get all data
+            var_data_df = river.io.d3d.get_all_data_points(data, var,time_step)           
+            TI_data_raw[var] = var_data_df 
+            TI_data= points.copy(deep=True)
+        
+        for var in TI_vars:    
+            TI_data[var] = interp.griddata(TI_data_raw[var][['x','y','z']],
+                                TI_data_raw[var][var], points[['x','y','z']])
+        
+        u_mag=river.io.d3d.unorm(TI_data['ucx'],TI_data['ucy'], TI_data['ucz'])
+        turbulent_intensity_expected= np.sqrt(2/3*TI_data['turkin1'])/u_mag
+       
+        
+        assert_array_almost_equal(TI.turbulent_intensity, turbulent_intensity_expected, decimal = 2)     
+       
 if __name__ == '__main__':
     unittest.main() 
 
